﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Runtime.InteropServices;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using ImageProcessor;
using ImageProcessor.Imaging.Filters.Photo;
using ImageProcessor.Imaging.Formats;

namespace ImageShop
{
    public partial class Form : System.Windows.Forms.Form
    {
        public List<string> fileNames = new List<string>();
        public int selectedIndex = 0;
        public string selectedImage = "";
        byte[] photoBytes = null;

        public Form()
        {
            InitializeComponent();
            textBoxOphaalmap.Text = Properties.Settings.Default.Ophaalmap;
            textBoxBewaarmap.Text = Properties.Settings.Default.Bewaarmap;
            textBoxDoelbestand.Text = Properties.Settings.Default.Doelbestand;
            comboBoxFilter.Items.Add("BlackWhite");
            comboBoxFilter.Items.Add("Comic");
            comboBoxFilter.Items.Add("Gotham");
            comboBoxFilter.Items.Add("GreyScale");
            comboBoxFilter.Items.Add("HiSatch");
            comboBoxFilter.Items.Add("Invert");
            comboBoxFilter.Items.Add("Lomograph");
            comboBoxFilter.Items.Add("LoSatch");
            comboBoxFilter.Items.Add("Polaroid");
            comboBoxFilter.Items.Add("Sepia");
            imageList.ImageSize = new Size(128, 72);
            populateImageList();
        }

        private void buttonKiesOphaalmap_Click(object sender, EventArgs e)
        {
            DialogResult result = folderBrowserDialog.ShowDialog();
            if (result == DialogResult.OK)
            {
                textBoxOphaalmap.Text = folderBrowserDialog.SelectedPath;
                Properties.Settings.Default.Ophaalmap = textBoxOphaalmap.Text;
                Properties.Settings.Default.Save();
            }
        }

        private void buttonKiesBewaarmap_Click(object sender, EventArgs e)
        {
            DialogResult result = folderBrowserDialog.ShowDialog();
            if (result == DialogResult.OK)
            {
                textBoxBewaarmap.Text = folderBrowserDialog.SelectedPath;
                Properties.Settings.Default.Bewaarmap = textBoxBewaarmap.Text;
                Properties.Settings.Default.Save();
                populateImageList();
            }
        }

        private void buttonKiesDoelbestand_Click(object sender, EventArgs e)
        {
            if (!String.IsNullOrEmpty(textBoxDoelbestand.Text))
            {
                openFileDialog.InitialDirectory = Path.GetDirectoryName(textBoxDoelbestand.Text);
            }
            openFileDialog.Title = "Kies doelbestand";
            openFileDialog.Filter = "(*.jpg, *.png)|*.jpg;*.png";
            DialogResult result = openFileDialog.ShowDialog();
            if (result == DialogResult.OK)
            {
                textBoxDoelbestand.Text = openFileDialog.FileName;
            }
            Properties.Settings.Default.Doelbestand = textBoxDoelbestand.Text;
            Properties.Settings.Default.Save();
        }

        private void buttonOphalenAfbeeldingen_Click(object sender, EventArgs e)
        {
            if (!String.IsNullOrEmpty(textBoxOphaalmap.Text))
            {
                openFileDialog.InitialDirectory = textBoxOphaalmap.Text;
            }
            openFileDialog.Title = "Ophalen afbeeldingen";
            openFileDialog.Filter = "(*.jpg, *.png)|*.jpg;*.png";
            DialogResult result = openFileDialog.ShowDialog();
            if (result == DialogResult.OK)
            {
                fileNames = new List<string>();
                imageList.Dispose();
                if (openFileDialog.FileNames != null)
                {
                    for (int i = 0; i < openFileDialog.FileNames.Length; i++)
                    {
                        saveImage(openFileDialog.FileNames[i]);
                    }
                    if (String.IsNullOrEmpty(textBoxOphaalmap.Text))
                    {
                        textBoxOphaalmap.Text = Path.GetDirectoryName(openFileDialog.FileNames[0]);
                    }
                }
                else
                {
                    saveImage(openFileDialog.FileName);
                    if (String.IsNullOrEmpty(textBoxOphaalmap.Text))
                    {
                        textBoxOphaalmap.Text = Path.GetDirectoryName(openFileDialog.FileName);
                    }
                }
                populateImageList();
            }
        }

        [DllImport("User32", CharSet = CharSet.Auto)]
        public static extern int SystemParametersInfo(int uiAction, int uiParam, string pvParam, uint fWinIni);

        private void buttonWallpaper_Click(object sender, EventArgs e)
        {
            if (!String.IsNullOrEmpty(textBoxDoelbestand.Text))
            {
                SystemParametersInfo(0x0014, 0, textBoxDoelbestand.Text, 0x0001);
            }
        }

        private void buttonLichter_Click(object sender, EventArgs e)
        {
            if (!String.IsNullOrEmpty(selectedImage))
            {
                using (MemoryStream inStream = new MemoryStream(photoBytes))
                {
                    using (MemoryStream outStream = new MemoryStream())
                    {
                        using (ImageFactory imageFactory = new ImageFactory(preserveExifData: true))
                        {
                            imageFactory.Load(inStream).Brightness(10).Save(outStream);
                            pictureBox.Image = Image.FromStream(outStream);
                            photoBytes = outStream.ToArray();
                        }
                    }
                }
            }
        }

        private void buttonDonkerder_Click(object sender, EventArgs e)
        {
            if (!String.IsNullOrEmpty(selectedImage))
            {
                using (MemoryStream inStream = new MemoryStream(photoBytes))
                {
                    using (MemoryStream outStream = new MemoryStream())
                    {
                        using (ImageFactory imageFactory = new ImageFactory(preserveExifData: true))
                        {
                            imageFactory.Load(inStream).Brightness(-10).Save(outStream);
                            pictureBox.Image = Image.FromStream(outStream);
                            photoBytes = outStream.ToArray();
                        }
                    }
                }
            }

        }

        private void buttonSpiegelen_Click(object sender, EventArgs e)
        {
            if (!String.IsNullOrEmpty(selectedImage))
            {
                using (MemoryStream inStream = new MemoryStream(photoBytes))
                {
                    using (MemoryStream outStream = new MemoryStream())
                    {
                        using (ImageFactory imageFactory = new ImageFactory(preserveExifData: true))
                        {
                            imageFactory.Load(inStream).Flip(false).Save(outStream);
                            pictureBox.Image = Image.FromStream(outStream);
                            photoBytes = outStream.ToArray();
                        }
                    }
                }
            }
        }

        private void buttonFlippen_Click(object sender, EventArgs e)
        {
            if (!String.IsNullOrEmpty(selectedImage))
            {
                using (MemoryStream inStream = new MemoryStream(photoBytes))
                {
                    using (MemoryStream outStream = new MemoryStream())
                    {
                        using (ImageFactory imageFactory = new ImageFactory(preserveExifData: true))
                        {
                            imageFactory.Load(inStream).Flip(true).Save(outStream);
                            pictureBox.Image = Image.FromStream(outStream);
                            photoBytes = outStream.ToArray();
                        }
                    }
                }
            }
        }

        private void buttonFilter_Click(object sender, EventArgs e)
        {
            if (!String.IsNullOrEmpty(selectedImage))
            {
                using (MemoryStream inStream = new MemoryStream(photoBytes))
                {
                    using (MemoryStream outStream = new MemoryStream())
                    {
                        using (ImageFactory imageFactory = new ImageFactory(preserveExifData: true))
                        {
                            switch (comboBoxFilter.SelectedIndex)
                            {
                                case 0:
                                    imageFactory.Load(inStream).Filter(MatrixFilters.BlackWhite).Save(outStream);
                                    break;
                                case 1:
                                    imageFactory.Load(inStream).Filter(MatrixFilters.Comic).Save(outStream);
                                    break;
                                case 2:
                                    imageFactory.Load(inStream).Filter(MatrixFilters.Gotham).Save(outStream);
                                    break;
                                case 3:
                                    imageFactory.Load(inStream).Filter(MatrixFilters.GreyScale).Save(outStream);
                                    break;
                                case 4:
                                    imageFactory.Load(inStream).Filter(MatrixFilters.HiSatch).Save(outStream);
                                    break;
                                case 5:
                                    imageFactory.Load(inStream).Filter(MatrixFilters.Invert).Save(outStream);
                                    break;
                                case 6:
                                    imageFactory.Load(inStream).Filter(MatrixFilters.Lomograph).Save(outStream);
                                    break;
                                case 7:
                                    imageFactory.Load(inStream).Filter(MatrixFilters.LoSatch).Save(outStream);
                                    break;
                                case 8:
                                    imageFactory.Load(inStream).Filter(MatrixFilters.Polaroid).Save(outStream);
                                    break;
                                case 9:
                                    imageFactory.Load(inStream).Filter(MatrixFilters.Sepia).Save(outStream);
                                    break;
                                default:
                                    imageFactory.Load(inStream).Save(outStream);
                                    break;
                            }
                            pictureBox.Image = Image.FromStream(outStream);
                            photoBytes = outStream.ToArray();
                        }
                    }
                }
            }
        }

        private void buttonHerstel_Click(object sender, EventArgs e)
        {
            if (!String.IsNullOrEmpty(selectedImage))
            {
                photoBytes = File.ReadAllBytes(selectedImage);
                pictureBox.Image = Image.FromStream(new MemoryStream(photoBytes));
            }
        }

        private void buttonBewaar_Click(object sender, EventArgs e)
        {
            if (!String.IsNullOrEmpty(textBoxDoelbestand.Text))
            {
                using (MemoryStream outStream = new MemoryStream(photoBytes))
                {
                    using (FileStream fileStream = File.Create(textBoxDoelbestand.Text, (int)outStream.Length))
                    {
                        byte[] bytesInStream = new byte[outStream.Length];
                        outStream.Read(bytesInStream, 0, bytesInStream.Length);
                        fileStream.Write(bytesInStream, 0, bytesInStream.Length);
                    }
                }
            }
        }

        private void listView_DoubleClick(object sender, EventArgs e)
        {
            for (int i = 0; i < listView.Items.Count; i++)
            {
                if (listView.Items[i] == listView.SelectedItems[0])
                {
                    selectedIndex = i;
                    selectedImage = fileNames[selectedIndex];
                    photoBytes = File.ReadAllBytes(selectedImage);
                    pictureBox.Image = Image.FromStream(new MemoryStream(photoBytes));
                }
            }
        }

        private void populateImageList()
        {
            if (!String.IsNullOrEmpty(textBoxBewaarmap.Text))
            {
                DirectoryInfo directory = new DirectoryInfo(textBoxBewaarmap.Text);
                FileInfo[] files = directory.GetFiles("*.png").Union(directory.GetFiles("*.jpg")).ToArray();
                if (files.Length > 0)
                {
                    fileNames = new List<string>();
                    imageList.Dispose();
                    foreach (FileInfo file in files)
                    {
                        addImage(textBoxOphaalmap.Text + "\\" + file.Name);
                    }
                    populateListView(true);
                }
            }
        }

        private void addImage(string fileName)
        {
            if (!String.IsNullOrWhiteSpace(fileName))
            {
                Image image = Image.FromStream(new MemoryStream(File.ReadAllBytes(fileName)));
                if ((image.Height == 1080) && (image.Width == 1920))
                {
                    fileNames.Add(fileName);
                    imageList.Images.Add(image);
                }
            }
        }

        private void populateListView(bool start)
        {
            listView.View = View.LargeIcon;
            listView.LargeImageList = imageList;
            listView.Clear();
            if (imageList.Images.Count > 0)
            {
                for (int i = 0; i < imageList.Images.Count; i++)
                {
                    ListViewItem item = new ListViewItem();
                    item.ImageIndex = i;
                    listView.Items.Add(item);
                }
                if (start == true)
                {
                    selectedImage = fileNames[0];
                }
                photoBytes = File.ReadAllBytes(selectedImage);
                pictureBox.Image = Image.FromStream(new MemoryStream(photoBytes));
            }
        }


        private void saveImage(string fileName)
        {
            if (!String.IsNullOrWhiteSpace(fileName))
            {
                Image image = Image.FromStream(new MemoryStream(File.ReadAllBytes(fileName)));
                if ((image.Height == 1080) && (image.Width == 1920))
                {
                    if (!String.IsNullOrEmpty(textBoxBewaarmap.Text))
                    {
                        string bestand = textBoxBewaarmap.Text + "\\" + Path.GetFileName(fileName);
                        image.Save(bestand);
                    }
                }
            }
        }

        private void Form_Load(object sender, EventArgs e)
        {

        }
    }
}
